/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@libsdl.org
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_syssem.c,v 1.5 2002/03/06 11:23:02 slouken Exp $";
#endif

/* Semaphore functions using the OS/2 API */

#include <stdio.h>
#include <stdlib.h>
#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_DOSSEMAPHORES
#include <os2.h>

#include "SDL_error.h"
#include "SDL_thread.h"
#include "SDL_timer.h"


struct SDL_semaphore {
        HMTX id;
        Uint32 value;
};


/* Create a semaphore */
DECLSPEC SDL_sem * SDLCALL SDL_CreateSemaphore(Uint32 initial_value)
{
        SDL_sem *sem;
        ULONG ulrc;

        /* Allocate sem memory */
        sem = (SDL_sem *)malloc(sizeof(*sem));
        if ( sem ) {
                /* Create the mutex semaphore */
                ulrc = DosCreateMutexSem(NULL,&(sem->id),0,TRUE);
                if ( ulrc ) {
                        SDL_SetError("Couldn't create semaphore");
                        free(sem);
                        sem = NULL;
                }
                sem->value = initial_value;
                DosReleaseMutexSem(sem->id);
        } else {
                SDL_OutOfMemory();
        }
        return(sem);
}

/* Free the semaphore */
DECLSPEC void SDLCALL SDL_DestroySemaphore(SDL_sem *sem)
{
        if ( sem ) {
                if ( sem->id ) {
                        DosCloseMutexSem(sem->id);
                        sem->id = 0;
                }
                free(sem);
        }
}

DECLSPEC int SDLCALL SDL_SemWaitTimeout(SDL_sem *sem, Uint32 timeout)
{
        ULONG ulrc, tick;

        if ( ! sem ) {
                SDL_SetError("Passed a NULL sem");
                return -1;
        }

        if ( timeout == SDL_MUTEX_MAXWAIT ) {
           while (1) {
              ulrc = DosRequestMutexSem(sem->id, SEM_INDEFINITE_WAIT);
              if (ulrc) {
                 /* if error waiting mutex */
                 SDL_SetError("DosRequestMutexSem() failed");
                 return -1;
              } else if (sem->value) {
                        sem->value--;
                        DosReleaseMutexSem(sem->id);
                        return 0;
                     } else {
                        DosReleaseMutexSem(sem->id);
                        /* continue waiting */
                     }
           }
        } else {
           tick = SDL_GetTicks();
           while ((SDL_GetTicks()-tick)<timeout) {
              ulrc = DosRequestMutexSem(sem->id, timeout);
              switch (ulrc) {
                 case NO_ERROR:
                    if (sem->value) {
                       sem->value--;
                       DosReleaseMutexSem(sem->id);
                       return 0;
                    } else {
                       DosReleaseMutexSem(sem->id);
                    }
                    break;
                 case ERROR_TIMEOUT:
                    return SDL_MUTEX_TIMEDOUT;
                    break;
                 default:
                    SDL_SetError("DosRequestMutexSem() failed");
                    return -1;
              }
           }
        }
        /* never reached */
        return -1;
}

DECLSPEC int SDLCALL SDL_SemTryWait(SDL_sem *sem)
{
        return SDL_SemWaitTimeout(sem, 0);
}

DECLSPEC int SDLCALL SDL_SemWait(SDL_sem *sem)
{
        return SDL_SemWaitTimeout(sem, SDL_MUTEX_MAXWAIT);
}

/* Returns the current count of the semaphore */
DECLSPEC Uint32 SDLCALL SDL_SemValue(SDL_sem *sem)
{
        if ( ! sem ) {
                SDL_SetError("Passed a NULL sem");
                return 0;
        }
        return sem->value;
}

DECLSPEC int SDLCALL SDL_SemPost(SDL_sem *sem)
{
        if ( ! sem ) {
                SDL_SetError("Passed a NULL sem");
                return -1;
        }
        if ( DosRequestMutexSem(sem->id,SEM_INDEFINITE_WAIT) ) {
                SDL_SetError("DosRequestMutexSem() failed");
                return -1;
        }
        sem->value++;
        DosReleaseMutexSem(sem->id);
        return 0;
}
