/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@libsdl.org
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_os2video.h,v 1.5 2002/08/20 00:20:06 slouken Exp $";
#endif

#ifndef _SDL_os2video_h
#define _SDL_os2video_h


// OS2 specific includes
#define INCL_TYPES
#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_DOSPROCESS
#define INCL_WIN
#define INCL_GPI
#include <os2.h>

#define INCL_GRE_DEVMISC
#include <pmddi.h>

#include <dive.h>

#include "gradd.h"
#include "additionals.h"

/* Hidden "this" pointer for the video functions */
#define _THIS	SDL_VideoDevice *_this

/* Private display data */
struct SDL_PrivateVideoData
{
  HMODULE       hmodVMAN;         // Handle of loaded VMAN.DLL module
  FNVMIENTRY    *pfnVMIEntry;     // The entry of VMAN.DLL
  LONG          NumVideoModes;    // Number of supported video modes
  PGDDMODEINFO  ModeInfo;         // List of all supported video modes
  GDDMODEINFO   DesktopModeInfo;  // One video mode, which is used for desktop
  PGDDMODEINFO  FullScreenModeInfo; // Video mode for fullscreen
  BOOL          fInFullScreenNow; // Flag to show if the Dive and the Desktop is
                                  // in Fullscreen or windowed mode now
  BOOL          fInFocus;         // Tells if the application is in focus or not
  
  HWND          hwndFrame, hwndClient;  // Window handle of frame and client
  HMTX          hmtxUseDive;      // Mutex semaphore to protect DIVE usage
  HDIVE         hDiveInst;        // Dive instance number
  PDIVE_CAPS    pDiveCaps;        // Dive capabilities
  ULONG         *pfccFormats;     // List of input and output formats accepted by Dive
  ULONG         ulImage;          // Handle of image buffer allocated by Dive
  int           iWidth;           // Width, height and FourCC of that buffer
  int           iHeight;
  ULONG         fccFourCC;
  void          *pBuffer;         // The buffer
  ULONG         ulScanLineBytes;  // and its parameter
  BOOL          fVRNDisabled;     // Flag to show if VRN is disabled or not
  BOOL          fFullScreenMode;  // Flag to show if the application is running in fullscreen mode or not.
                                  // Note, that it doesn't mean that the desktop is in fullscreen mode now,
                                  // because it only shows that when the application gets focus, it should
                                  // run in fullscreen mode (or not).
                                  // To check the actual mode, see fInFullscreenNow!
  SDL_Rect      **pListModesResult;// Allocated memory to return list of modes for os2video_ListModes() API

  int           iPMThreadStatus;  // 0: Not running
                                  // 1: Running
                                  // Other: Not running, had an error
  TID           tidPMThread;      // Thraead ID of PM Thread
};

#endif /* _SDL_os2video_h */
